@@ -1,10 +1,21 @@
 from functools import wraps
-from logging import basicConfig, getLogger
+from logging import basicConfig
+from logging import getLogger
 from time import sleep
-from typing import Any, List, Union, Tuple, Sequence, Sized
+from typing import Any
+from typing import List
+from typing import Sequence
+from typing import Sized
+from typing import Tuple
+from typing import Union
+from warnings import warn
 
-from numpy import array, nan, nan_to_num, ndarray
-from pandas import DataFrame, Series
+from numpy import array
+from numpy import nan
+from numpy import nan_to_num
+from numpy import ndarray
+from pandas import DataFrame
+from pandas import Series
 from ta.volatility import AverageTrueRange
 
 PREDICT_TYPE: type = int
@@ -21,19 +32,19 @@ TEXT_COLOR: str = 'white'
 TIME_TITLE: str = 'T I M E'
 
 DEPOSIT_TITLE: str = 'M O N E Y S'
-DEPOSIT_NAME: str = 'deposit (start: {})'  # .format(Trader.deposit_history[0])
+DEPOSIT_NAME: str = 'deposit (start: {0})'  # .format(Trader.deposit_history[0])
 DEPOSIT_COLOR: str = 'white'
-DEPOSIT_WIDTH: float = 1.0
+DEPOSIT_WIDTH: float = 2.0
 DEPOSIT_ALPHA: float = 1.0
 
 RETURNS_TITLE: str = 'R E T U R N S'
 RETURNS_NAME: str = 'returns'
 RETURNS_COLOR: str = DEPOSIT_COLOR
-RETURNS_WIDTH: float = 1.0
+RETURNS_WIDTH: float = 2.0
 RETURNS_ALPHA: float = 1.0
 
 DATA_TITLE: str = 'D A T A'
-DATA_NAME: str = '{} {}'  # '{} {}'.format(Trader.ticker, Trader.interval)
+DATA_NAME: str = '{0} {1}'  # '{} {}'.format(Trader.ticker, Trader.interval)
 DATA_UP_COLOR = 'green'
 DATA_DOWN_COLOR = 'red'
 DATA_ALPHA: float = 1.0
@@ -78,7 +89,7 @@ TRADE_MARKER_EXIT_ALPHA: float = 1.0
 
 TICKER_PATTERN: str = r'[A-Z]+/[A-Z]+'
 
-WAIT_SUCCESS_SLEEP: float = 15.0
+WAIT_SUCCESS_SLEEP: float = 1.0
 WAIT_SUCCESS_PRINT: bool = True
 WAIT_SUCCESS_USE: bool = True
 
@@ -98,11 +109,16 @@ MA_SLOW_WIDTH: float = 4.5
 MA_SLOW_ALPHA: float = 1.0
 
 ICHIMOKU_LINES_WIDTH: float = 2.0
-SENKOU_SPAN_A_COLOR: str = '#ff0000'
-SENKOU_SPAN_A_NAME: str = 'senkou span a'
+
+ICHIMOKU_CLOUD_COLOR: str = 'rgb(200,250,10)'
+
 SENKOU_SPAN_B_NAME: str = 'senkou span b'
-ICHIMOKU_CLOUD_COLOR: str = 'rgb(0,250,250)'
-ICHIMOKU_CLOUD_ALPHA: float = 0.4
+SENKOU_SPAN_B_COLOR: str = '#55ff00'
+SENKOU_SPAN_B_ALPHA: float = 0.4
+
+SENKOU_SPAN_A_NAME: str = 'senkou span a'
+SENKOU_SPAN_A_COLOR: str = '#ff0000'
+SENKOU_SPAN_A_ALPHA: float = 0.4
 
 SAR_UP_NAME: str = 'SAR up'
 SAR_UP_COLOR: str = '#ffff00'
@@ -115,7 +131,7 @@ SAR_DOWN_WIDTH: float = 2
 SAR_DOWN_ALPHA: float = 1.0
 
 ST_UP_NAME: str = 'SuperTrend up'
-ST_UP_COLOR: str = '#ffff00'
+ST_UP_COLOR: str = '#ff0000'
 ST_UP_WIDTH: float = 2
 ST_UP_ALPHA: float = 1.0
 
@@ -139,7 +155,13 @@ LOWER_BB_COLOR: str = '#00ff7b'
 LOWER_BB_WIDTH: float = 2
 LOWER_BB_ALPHA: float = 1.0
 
-__version__: str = "6.0.0"
+INFO_TEXT: str = """losses: {}
+trades: {}
+profits: {}
+mean year percentage profit: {}%
+winrate: {}%"""  # .format(Trader.losses, Trader.trades, Trader.profits, Trader.year_profit, Trader.winrate)
+
+__version__: str = "6.3.8"
 __author__: str = 'Vlad Kochetov'
 __credits__: List[str] = [
     "Hemerson Tacon -- Stack overflow",
@@ -393,7 +415,7 @@ def wait_success(func):
     return checker
 
 
-def root(x: float, pwr: float = 2) -> float:
+def root(x: float, pwr: float = 2.0) -> float:
     return x ** (1 / pwr)
 
 
@@ -433,3 +455,31 @@ def get_diff(price: float,
 
     elif signal == SELL and low <= take_profit:
         return take_profit - price
+
+
+def make_multi_deal_returns(converted_returns: CONVERTED_TYPE_LIST) -> Tuple[PREDICT_TYPE_LIST, List[int]]:
+    if EXIT in converted_returns:
+        warn('The use of utils.EXIT is deprecated in this type of strategy. If utils.EXIT is the first item in the sequence, you can replace it with np.nan.')
+    result_credlev: List[int] = []
+    result_returns: PREDICT_TYPE_LIST = [BUY] * len(converted_returns)
+    flag_lev: int = 0
+    if converted_returns[0] is nan:
+        converted_returns[0] = EXIT
+    ret: CONVERTED_TYPE
+    for ret in converted_returns:
+        if ret is not nan:
+            if ret is BUY:
+                flag_lev += 1
+            elif ret is SELL:
+                flag_lev -= 1
+        result_credlev.append(flag_lev)
+    e: int
+    lev: int
+    for e, lev in enumerate(result_credlev):
+        if lev < 0:
+            result_credlev[e] = -lev
+            result_returns[e] = SELL
+        elif lev == 0:
+            result_credlev[e] = 1
+            result_returns[e] = EXIT
+    return result_returns, result_credlev
